program test3;

(*********************************************************************
 *
 * This programs lists all U12 devices and gets their properties
 * using the libljacklm library.
 *
 * kms
 * march 2016
 *
 *********************************************************************)

uses
  ctypes;

const
{$IFDEF MSWINDOWS}
  libljacklmName = 'libljacklm.dll';
{$ENDIF MSWINDOWS}
{$IFDEF LINUX}
  libljacklmName = 'libljacklm.so.0';
{$ENDIF LINUX}
{$IFDEF DARWIN}
  libljacklmName = 'libljacklm.dylib';
  {$linklib libljacklm}
{$ENDIF DARWIN}

type
  tCalMatrix     = array[1..20] of clong;
  tCalMatrixList = array[1..127] of tCalMatrix;
  t127Integers   = array[1..127] of clong;
  tErrorString   = array[1..50] of char;

var
  errorNumber, i, j: integer;
  productIDList: t127Integers; 
  serialnumList: t127Integers;
  localIDList: t127Integers;
  powerList: t127Integers;
  calMatrixList: tCalMatrixList;
  numberFound: clong;
  fcddMaxSize: clong;
  hvcMaxSize: clong;
  errorString: tErrorString;

function ListAll(var productIDList: t127Integers; 
                 var serialnumList: t127Integers;
                 var localIDList: t127Integers;
                 var powerList: t127Integers;
                 var calMatrixList: tCalMatrixList;
                 var numberFound: clong;
                 var fcddMaxSize: clong;
                 var hvcMaxSize: clong): clong;
         cdecl; external libljacklmName;

procedure GetErrorString (errorcode: clong;
                          var errorString: tErrorString);
          cdecl; external libljacklmName;

begin
  writeln ('+-----------------------+');
  writeln ('| List all U12 devices. |');
  writeln ('+-----------------------+');
  errorNumber := ListAll(productIDList,
                         serialnumList,
                         localIDList,
                         powerList,
                         calMatrixList,
                         numberFound,
                         fcddMaxSize,
                         hvcMaxSize);

  if errorNumber <> 0 then
  begin
    GetErrorString(errorNumber, errorString);
    writeln ('ListAll error ', errorNumber, ': ', errorString);
    exit;
  end;
  
  writeln ('ListAll found ', numberFound, ' U12 device(s).');
  if numberFound > 0 then
  begin
    writeln ('Device informations:');
    writeln ('| Product ID | Serial Number | LocalID | Power Allowance |');
    
    for i := 1 to numberFound do
    begin
      writeln ('| ', productIDList[i]:10, ' | ', serialnumList[i]:13, ' | ', localIDList[i]:7, ' | ', powerList[i]:15, ' | ');
      writeln ('| Calibration Constants:');
      write   ('| ');
      for j := 1 to 20 do
        write (calMatrixList[i,j], ' ');
      writeln;
    end;
  end;
  writeln ('+-----------------------------------+');
  writeln ('| Finished listing all U12 devices. |');
  writeln ('+-----------------------------------+');
end.