program test5;

(*********************************************************************
 *
 * This programs sets analog outputs of a U12 device
 * using the libljacklm library.
 *
 * kms
 * march 2016
 *
 *********************************************************************)

uses
  ctypes,
  SysUtils;

const
{$IFDEF MSWINDOWS}
  libljacklmName = 'libljacklm.dll';
{$ENDIF MSWINDOWS}
{$IFDEF LINUX}
  libljacklmName = 'libljacklm.so.0';
{$ENDIF LINUX}
{$IFDEF DARWIN}
  libljacklmName = 'libljacklm.dylib';
  {$linklib libljacklm}
{$ENDIF DARWIN}
 
const
  numChannels = 2;  // Using 2 channels

type
  tErrorString = array[1 .. 50]  of char;
  t2Integers   = array[1 .. numChannels] of clong;
  tVoltages    = array[1 .. numChannels + 1] of single; // exact length does not work!!

var
  errorNumber, i, demo, trisD, trisIO, stateD, stateIO: clong;
  updateDigital, resetCounter, count, idnum: clong;
  analogOut0, analogOut1: real;
  updateIO, ledOn, disableCal, overVoltage: clong;
  channels: t2Integers;
  gains:    t2Integers;
  Voltages: tVoltages;
  errorString: tErrorString;

function GetDriverVersion: cfloat;
         cdecl; external libljacklmName;
         
function GetFirmwareVersion(var idnum: clong): cfloat;
         cdecl; external libljacklmName;

procedure GetErrorString (errorcode: clong;
                          var errorString: tErrorString);
          cdecl; external libljacklmName;

function AOUpdate(var idnum: clong;
                  demo, trisD, trisIO: clong;
                  var stateD: clong;
                  var stateIO: clong;
                  updateDigital, resetCounter: clong;
                  var count: clong;
                  analogOut0, analogOut1: single): clong;
          cdecl; external libljacklmName;
                        
function AISample(var idnum: clong;
                  demo: clong;
                  var stateIO: clong;
                  updateIO, ledOn, numChannels: clong;
                  var channels: t2Integers;
                  var gains: t2Integers;
                  disableCal: clong;
                  var overVoltage: clong;
                  var voltages: tVoltages): clong;
          cdecl; external libljacklmName;
                        
procedure handleError (errorNumber: integer; functionName: tErrorString);
begin
  if errorNumber <> 0 then
  begin
    GetErrorString(errorNumber, errorString);
    writeln (functionName, ' error ', errorNumber, ': ', errorString);
    exit;
  end;
end;

begin
  writeln ('+-----------------------+');
  writeln ('| List all U12 devices. |');
  writeln ('+-----------------------+');

  writeln('Library version: ', GetDriverVersion:7:4);

  idnum := -1; // -1 means take the first device  
  writeln('*** Getting firmware version does intialization and takes a moment. ***');
  writeln('U12 Firmware version: ', GetFirmwareVersion(idnum):7:4);

  // AOUpdate specific paramters
  demo    := 0;   // Normal operations
  trisD   := 0;  // Directions for D0-D15
  trisIO  := 0;  // Directions for IO0-IO3
  stateD  := 0;  // Output states for D0-D15
  stateIO := 0;   // Output states for IO0-IO3
  updateDigital := 0;  // Tris and state values are only being read
  resetCounter  := 0;  // Not resetting counter
  count := 0;  // Returned current count value
  analogOut0     := 0.8;  // Voltage for AO0
  analogOut1     := 3.5;  // Voltage for AO1
  
  writeln('Ramping AO0 from 0 to 2.5 V and AO1 from 5.0 to 2.5 V');

  for i := 0 to 25 do
  begin
    write ('*');
    analogOut0 := i / 10.0;
    analogOut1 := 5.0 - i / 10.0;
    errorNumber := AOUpdate(idnum, demo, trisD, trisIO, stateD,
          stateIO, updateDigital, resetCounter, count, analogOut0,
          analogOut1);
    sleep(300);  // milliseconds
  end;
  writeln;

  // AISample specific parameters
  updateIO    := 0;  // State values are only being read
  ledOn       := 0;  // Turning LED on
  disableCal  := 0;  // Will apply calibration constants
  overVoltage := 0;  // Returns if overvoltage was detected (>1)
  channels[1] := 0;
  channels[2] := 1;  // Reading AI0 and AI1
  gains[1]    := 0;   // Gains.  Does not matter in this case since we are
  gains[2]    := 0;   // performing single ended readings.
  voltages[1] := 0.0;  // Returned voltage readings.  Pass an array of all zeros.
  voltages[2] := 0.0;

  writeln('Measure AI0 and AI1');

  errorNumber := AISample(idnum, demo, stateIO, updateIO, ledOn, 
          numChannels, channels, gains, disableCal, overVoltage,
          voltages);
  handleError(errorNumber, 'AISample');

  writeln('AI', channels[1], ': ', voltages[1]:5:3, ' V');
  writeln('AI', channels[2], ': ', voltages[2]:5:3, ' V');
  
  writeln ('+-----------------------------------+');
  writeln ('| Finished listing all U12 devices. |');
  writeln ('+-----------------------------------+');
end.