program test7;

(*********************************************************************
 *
 * This programs loads the libljacklm library dynamically at runtime.
 *
 * kms
 * may 2016
 *
 *********************************************************************)

uses
  ctypes,
  DynLibs;

const
{$IFDEF MSWINDOWS}
  libljacklmName = 'libljacklm.dll';
{$ENDIF MSWINDOWS}
{$IFDEF LINUX}
  libljacklmName = 'libljacklm.so.0';
{$ENDIF LINUX}
{$IFDEF DARWIN}
  libljacklmName = 'libljacklm.dylib';
  {$linklib libljacklm}
{$ENDIF DARWIN}
 
type
  tErrorString = array[1 .. 50]  of char;
  TGetDriverVersion =   function: cfloat; cdecl;
  TGetFirmwareVersion = function (var idnum: clong): cfloat; cdecl;
  TGetErrorString =     procedure (errorcode: clong; var errorString: tErrorString); cdecl;

var
  LibHandle: TLibHandle;
  idnum: clong;
  errorString: tErrorString;

  GetDriverVersion:   TGetDriverVersion;
  GetFirmwareVersion: TGetFirmwareVersion;
  GetErrorString:     TGetErrorString;
  
begin
  writeln ('+----------------------------------------+');
  writeln ('| Load the libljacklm library at runtime |');
  writeln ('+----------------------------------------+');
  
  LibHandle := LoadLibrary(libljacklmName);
  
  if LibHandle = NilHandle then
  begin
    errorString := GetLoadErrorStr;
    writeln ('Loading libljacklm failed. Error desciption: ', errorString);
    exit
  end;
  
  GetDriverVersion := TGetDriverVersion(GetProcedureAddress(LibHandle, 'GetDriverVersion'));
  if GetDriverVersion = nil then
  begin
    errorString := GetLoadErrorStr;
    writeln ('Getting the address of GetDriverVersion failed.');
    writeln ('Error desciption: ', errorString);
    exit
  end;
  
  GetFirmwareVersion := TGetFirmwareVersion(GetProcedureAddress(LibHandle, 'GetFirmwareVersion'));
  if GetFirmwareVersion = nil then
  begin
    errorString := GetLoadErrorStr;
    writeln ('Getting the address of GetFirmwareVersion failed.');
    writeln ('Error desciption: ', errorString);
    exit
  end;

  GetErrorString := TGetErrorString(GetProcedureAddress(LibHandle, 'GetErrorString'));
  if GetErrorString = nil then
  begin
    errorString := GetLoadErrorStr;
    writeln ('Getting the address of GetErrorString failed.');
    writeln ('Error desciption: ', errorString);
    exit
  end;

	writeln('Library version: ', GetDriverVersion:7:4);

	idnum := -1; // -1 means take the first device  
	writeln('*** Getting firmware version does intialization and takes a moment. ***');
	writeln('U12 Firmware version: ', GetFirmwareVersion(idnum):7:4);
  
  if UnloadLibrary(LibHandle) then
    writeln ('Library successfully unloaded')
  else
    writeln ('Unloading the library failed');

  writeln ('+----------------------------------------------------+');
  writeln ('| Finished loading the libljacklm library at runtime |');
  writeln ('+----------------------------------------------------+');
end.